@@ -1,5 +1,6 @@
 "use strict";
 
+var inspect = require("util").inspect;
 var sinon = require("sinon");
 var referee = require("@sinonjs/referee");
 var formatio = require("@sinonjs/formatio");
@@ -95,21 +96,6 @@ describe("referee-sinon", function() {
     });
 
     describe("assertions", function() {
-        it("formats assert messages through formatio", function() {
-            sinon.stub(formatio, "ascii").returns("I'm the object");
-            var message;
-            var spy = sinon.spy();
-            spy({ id: 42 });
-
-            try {
-                assert.calledWith(spy, 3);
-            } catch (e) {
-                message = e.message;
-            }
-
-            assert.match(message, "I'm the object");
-        });
-
         describe("calledWith", function() {
             it("fails when not called with spy", requiresSpy("calledWith"));
 
@@ -129,8 +115,9 @@ describe("referee-sinon", function() {
                     throw new Error("Exception expected");
                 } catch (e) {
                     var message =
-                        "[assert.calledWith] Expected function " +
-                        "spy() {} to be called with arguments null, 2, 2" +
+                        "[assert.calledWith] Expected " +
+                        inspect(spy) +
+                        " to be called with arguments null, 2, 2" +
                         "\n    spy(null, 1, 2)";
                     assert.match(e.message, message);
                 }
@@ -152,12 +139,30 @@ describe("referee-sinon", function() {
                     throw new Error("Exception expected");
                 } catch (e) {
                     var message =
-                        "[assert.calledWith] Expected function " +
-                        "fake() {} to be called with arguments null, 2, 2" +
+                        "[assert.calledWith] Expected " +
+                        inspect(fake) +
+                        " to be called with arguments null, 2, 2" +
                         "\n    fake(null, 1, 2)";
                     assert.match(e.message, message);
                 }
             });
+
+            it("formats error argument", function() {
+                var fake = sinon.fake();
+                fake(new Error());
+
+                try {
+                    assert.calledWith(fake, null);
+                    throw new Error("Exception expected");
+                } catch (e) {
+                    var message =
+                        "[assert.calledWith] Expected " +
+                        inspect(fake) +
+                        " to be called with arguments null" +
+                        "\n    fake(Error)";
+                    assert.match(e.message, message);
+                }
+            });
         });
 
         describe("calledWithExactly", function() {
@@ -183,7 +188,8 @@ describe("referee-sinon", function() {
                 } catch (e) {
                     var message =
                         "[assert.calledWithExactly] Expected " +
-                        "function spy() {} to be called with exact " +
+                        inspect(spy) +
+                        " to be called with exact " +
                         "arguments null, 2, 2\n    spy(null, 1, 2)";
                     assert.match(e.message, message);
                 }
@@ -223,7 +229,8 @@ describe("referee-sinon", function() {
                 } catch (e) {
                     var message =
                         "[assert.calledWithMatch] Expected " +
-                        "function spy() {} to be called with matching " +
+                        inspect(spy) +
+                        " to be called with matching " +
                         "arguments { check: 321 }\n    spy({ check: 123 })";
                     assert.match(e.message, message);
                 }
@@ -240,6 +247,77 @@ describe("referee-sinon", function() {
             });
         });
 
+        describe("calledOnceWithMatch", function() {
+            it(
+                "fails when not called with spy",
+                requiresSpy("calledOnceWithMatch")
+            );
+
+            it("passes when spy is passed matching object", function() {
+                var spy = sinon.spy();
+                spy({
+                    check: 123,
+                    color: "#fff"
+                });
+
+                assert.calledOnceWithMatch(spy, { check: 123 });
+            });
+
+            it("fails when spy is passes non-matching object", function() {
+                var spy = sinon.spy();
+                spy({
+                    check: 123,
+                    color: "#fff"
+                });
+
+                refute.calledOnceWithMatch(spy, { check: 321 });
+            });
+
+            it("fails when spy was called more than once", function() {
+                var spy = sinon.spy();
+                spy({
+                    check: 123,
+                    color: "#fff"
+                });
+                spy({
+                    check: 321,
+                    color: "#eee"
+                });
+
+                refute.calledOnceWithMatch(spy, { check: 123 });
+            });
+
+            context(
+                "when called more than once with same arguments",
+                function() {
+                    it("fails", function() {
+                        var spy = sinon.spy();
+                        spy("hello", "Hey");
+                        spy("hello", "Hey");
+
+                        refute.calledOnceWithMatch(spy, "hello", "Hey");
+                    });
+                }
+            );
+
+            it("formats message nicely", function() {
+                var spy = sinon.spy();
+                spy(null, 1, 2);
+
+                try {
+                    assert.calledOnceWithMatch(spy, null, 2, 2);
+                    throw new Error("Exception expected");
+                } catch (e) {
+                    var message =
+                        "[assert.calledOnceWithMatch] Expected " +
+                        inspect(spy) +
+                        " to be called once with matching arguments " +
+                        "null, 2, 2\n    spy(null, 1, 2)";
+                    assert.match(e.message, message);
+                }
+            });
+        });
+
         describe("alwaysCalledWithMatch", function() {
             it(
                 "fails when not called with spy",
@@ -271,11 +349,12 @@ describe("referee-sinon", function() {
                 } catch (e) {
                     var message =
                         "[assert.alwaysCalledWithMatch] Expected " +
-                        "function spy() {} to always be called with " +
+                        inspect(spy) +
+                        " to always be called with " +
                         "matching arguments { check: 321 }";
-                    assert.match(e.message, message);
                     var lines = e.message.split("\n");
                     assert.equals(lines.length, 3);
+                    assert.equals(lines[0], message);
                     assert.match(lines[1], "    spy({ check: 123 })");
                     assert.match(lines[2], "    spy({ check: 321 })");
                 }
@@ -293,14 +372,17 @@ describe("referee-sinon", function() {
             });
 
             it("formats message nicely", function() {
+                var spy = sinon.spy();
+
                 try {
-                    assert.calledOnce(sinon.spy());
+                    assert.calledOnce(spy);
                     throw new Error("Exception expected");
                 } catch (e) {
                     var message =
-                        "[assert.calledOnce] Expected function " +
-                        "spy() {} to be called once but was called 0 times";
-                    assert.match(e.message, message);
+                        "[assert.calledOnce] Expected " +
+                        inspect(spy) +
+                        " to be called once but was called 0 times";
+                    assert.equals(e.message, message);
                 }
             });
         });
@@ -325,10 +407,11 @@ describe("referee-sinon", function() {
                     throw new Error("Exception expected");
                 } catch (e) {
                     var message =
-                        "[assert.calledTwice] Expected function " +
-                        "spy() {} to be called twice but was called once";
+                        "[assert.calledTwice] Expected " +
+                        inspect(spy) +
+                        " to be called twice but was called once";
                     assert.match(e.message, message);
-                    assert.match(e.message, "spy(null, Hey)");
+                    assert.match(e.message, "spy(null, 'Hey')");
                 }
             });
         });
@@ -355,11 +438,12 @@ describe("referee-sinon", function() {
                     throw new Error("Exception expected");
                 } catch (e) {
                     var message =
-                        "[assert.calledThrice] Expected function " +
-                        "spy() {} to be called thrice but was called twice";
+                        "[assert.calledThrice] Expected " +
+                        inspect(spy) +
+                        " to be called thrice but was called twice";
                     assert.match(e.message, message);
-                    assert.match(e.message, "spy(null, Hey)");
-                    assert.match(e.message, "spy(null, Ho)");
+                    assert.match(e.message, "spy(null, 'Hey')");
+                    assert.match(e.message, "spy(null, 'Ho')");
                 }
             });
         });
@@ -377,12 +461,16 @@ describe("referee-sinon", function() {
             });
 
             it("formats message nicely", function() {
+                var spy = sinon.spy();
+
                 try {
-                    assert.callCount(sinon.spy(), 1);
+                    assert.callCount(spy, 1);
                     throw new Error("Exception expected");
                 } catch (e) {
                     var message =
-                        "[assert.callCount] Expected function spy() {} to be called exactly 1 times, but was called 0 times";
+                        "[assert.callCount] Expected " +
+                        inspect(spy) +
+                        " to be called exactly 1 times, but was called 0 times";
                     assert.match(e.message, message);
                 }
             });
@@ -399,13 +487,16 @@ describe("referee-sinon", function() {
             });
 
             it("formats message", function() {
+                var spy = sinon.spy();
+
                 try {
-                    assert.called(sinon.spy());
+                    assert.called(spy);
                     throw new Error("Exception expected");
                 } catch (e) {
                     var message =
-                        "[assert.called] Expected function spy() " +
-                        "{} to be called at least once but was never " +
+                        "[assert.called] Expected " +
+                        inspect(spy) +
+                        " to be called at least once but was never " +
                         "called";
                     assert.equals(e.message, message);
                 }
@@ -425,15 +516,17 @@ describe("referee-sinon", function() {
             });
 
             it("formats message", function() {
-                try {
-                    var spy = sinon.spy();
-                    spy();
+                var spy = sinon.spy();
+                spy();
 
+                try {
                     assert.calledWithNew(spy);
                     throw new Error("Exception expected");
                 } catch (e) {
                     var message =
-                        "[assert.calledWithNew] Expected function spy() {} to be called with 'new' at least once but was never called with 'new'";
+                        "[assert.calledWithNew] Expected " +
+                        inspect(spy) +
+                        " to be called with 'new' at least once but was never called with 'new'";
                     assert.equals(e.message, message);
                 }
             });
@@ -468,16 +561,18 @@ describe("referee-sinon", function() {
             });
 
             it("formats message", function() {
-                try {
-                    var spy = sinon.spy();
-                    spy();
-                    spy();
+                var spy = sinon.spy();
+                spy();
+                spy();
 
+                try {
                     assert.alwaysCalledWithNew(spy);
                     throw new Error("Exception expected");
                 } catch (e) {
                     var message =
-                        "[assert.alwaysCalledWithNew] Expected function spy() {} to always be called with 'new'";
+                        "[assert.alwaysCalledWithNew] Expected " +
+                        inspect(spy) +
+                        " to always be called with 'new'";
                     assert.equals(e.message, message);
                 }
             });
@@ -540,8 +635,9 @@ describe("referee-sinon", function() {
                     throw new Error("Exception expected");
                 } catch (e) {
                     var message =
-                        "[assert.calledOn] Expected function spy() " +
-                        "{} to be called with { id: 12 } as this but was " +
+                        "[assert.calledOn] Expected " +
+                        inspect(spy) +
+                        " to be called with { id: 12 } as this but was " +
                         "called on { id: 42 }";
                     assert.equals(e.message, message);
                 }
@@ -580,8 +676,9 @@ describe("referee-sinon", function() {
                     throw new Error("Exception expected");
                 } catch (e) {
                     var message =
-                        "[assert.alwaysCalledOn] Expected function " +
-                        "spy() {} to always be called with { id: 12 } as " +
+                        "[assert.alwaysCalledOn] Expected " +
+                        inspect(spy) +
+                        " to always be called with { id: 12 } as " +
                         "this but was called on { id: 42 }";
                     assert.equals(e.message, message);
                 }
@@ -612,7 +709,8 @@ describe("referee-sinon", function() {
                 } catch (e) {
                     var message =
                         "[assert.alwaysCalledWith] Expected " +
-                        "function spy() {} to always be called with " +
+                        inspect(spy) +
+                        " to always be called with " +
                         "arguments 12\n    spy(42)";
                     assert.match(e.message, message);
                 }
@@ -642,7 +740,8 @@ describe("referee-sinon", function() {
                 } catch (e) {
                     var message =
                         "[assert.alwaysCalledWithExactly] Expected " +
-                        "function spy() {} to always be called with " +
+                        inspect(spy) +
+                        " to always be called with " +
                         "exact arguments null, 2, 2\n    spy(null, 1, 2)";
                     assert.match(e.message, message);
                 }
@@ -685,7 +784,8 @@ describe("referee-sinon", function() {
                 } catch (e) {
                     var message =
                         "[assert.calledOnceWithExactly] Expected " +
-                        "function spy() {} to be called once with exact arguments " +
+                        inspect(spy) +
+                        " to be called once with exact arguments " +
                         "null, 2, 2\n    spy(null, 1, 2)";
                     assert.match(e.message, message);
                 }
@@ -712,9 +812,10 @@ describe("referee-sinon", function() {
                     throw new Error("Exception expected");
                 } catch (e) {
                     var message =
-                        "[assert.threw] Expected function spy() " +
-                        "{} to throw an exception";
-                    assert.match(e.message, message);
+                        "[assert.threw] Expected " +
+                        inspect(spy) +
+                        " to throw an exception";
+                    assert.equals(e.message, message);
                 }
             });
 
@@ -748,9 +849,10 @@ describe("referee-sinon", function() {
                     throw new Error("Exception expected");
                 } catch (e) {
                     var message =
-                        "[assert.alwaysThrew] Expected function " +
-                        "spy() {} to always throw an exception";
-                    assert.match(e.message, message);
+                        "[assert.alwaysThrew] Expected " +
+                        inspect(spy) +
+                        " to always throw an exception";
+                    assert.equals(e.message, message);
                 }
             });
         });
@@ -792,8 +894,9 @@ describe("referee-sinon", function() {
                     throw new Error("Exception expected");
                 } catch (e) {
                     var message =
-                        "[assert.calledOnceWith] Expected function " +
-                        "spy() {} to be called once with arguments 12\n" +
+                        "[assert.calledOnceWith] Expected " +
+                        inspect(spy) +
+                        " to be called once with arguments 12\n" +
                         "    spy(42)";
                     assert.match(e.message, message);
                 }
